#!/bin/bash

############# Script settings ###################

#To blacklist a plugin documentation, add plugin's names in lower case - separated by space 
blacklist=( check_timeout )

#The install directory of plugins
pluginsDir="/usr/lib/nagios/plugins/"
#The directory where wiki pages are saved with the namespace "plugins"
wikiDir="/var/www/dokuwiki/data/pages/plugins/" 
#The log file path
logFile="/var/log/updatefanwikierrors"
#verbose mode
verbose=0
# All created pages
pages=""
#################################################



##this function returns the list of installed plugins
getInstalledPlugins () {

installedPlug=( $(find $pluginsDir -name "check*") )
echo ${installedPlug[@]}

}

##this fnction returns the name of the plugin in lower case and without extension
getPluginName(){

pluginName=$(echo $1 | awk -F "/" '{print tolower($NF)}' | awk -F "." '{print $1}')
echo $pluginName

}


##this function returns the list of the already documented plugins
getDocumentedPlugins () {

documentedPlug=( $(ls $wikiDir | grep txt | sed '/txt./d' | awk -F "." '{print tolower($1)}') )
echo ${documentedPlug[@]}

}


##this function check the presence of the plugin in the blacklist
checkBlacklist() {

        i=0
        tablenght=${#blacklist[@]}

                while [ $i -ne $tablenght ]
                do

                        if [ $plugname = ${blacklist[$i]} ]; then
                        echo "kickban"
                        break
                        fi

        let i+=1
        done

}

printHelp() {
	echo
	echo "updateFanWiki.sh - This script generates the wiki documentation pages of the nagios's plugins"
	echo
	echo "=== Options ==="
	echo
	echo "-a"
	echo "delete the previous documentation and regenerate it from scratch "
	echo
	echo "-d "pluginName""	
	echo "delete the wiki page of the plugin"
	echo
	echo  "-h"
	echo "print this help "
	echo
	echo "-r "pluginName""	
	echo "delete the wiki page of the plugin and recreate it"
	echo
	echo "-u"
	echo "Generate or update the wiki"

}	
	






## This function create the wiki pages
createWikiPages () {


	if test -d $wikiDir; then
	echo "The plugins documentation will be updated"

	else
	echo "The plugins documentation will be created"
	mkdir $wikiDir
	fi


	for args in $(getInstalledPlugins)
	do 
		plugname=$(getPluginName $args) 		
		
		

		if [ "$(checkBlacklist)" != "kickban" ]; then
		        pages="$pages $plugname"
			#test if the wiki page was created before and do nothing
			if test -e $wikiDir$plugname.txt; then
         		        
				if [ $verbose -eq 1 ]; then
				echo "|||||||||||||||||||||||||||| The plugin $plugname already documented ||||||||||||||||||||"
				echo
				fi
			else
				
				if [ $verbose -eq 1 ]; then
				echo "----------=============== Creating $plugname wiki page ===============----------"
				fi
	
				helptmp=$($args --help 2>/dev/null)
				versiontmp=$($args --version 2>/dev/null)
				#test if helptmp is not empty
				if test -n "$helptmp"; then

					cat > $wikiDir$plugname.txt <<-EOT
					====== $plugname ======
					===== Version =====					
					<code>$versiontmp</code>
					===== Samples =====
					Print help:
					<code>$args --help</code>					
					===== Help =====
					<code>$helptmp</code>
					EOT
					
					if [ $verbose -eq 1 ]; then
					echo "#====== #################### $plugname wiki page created ####################" 
					echo
					fi
					let numberOfPages+=1

				else

					if [ $verbose -eq 1 ]; then
					echo
					echo "@@@@@@@@@@@@@@@@@@@@@@ $plugname wiki page not created @@@@@@@@@@@@@@@@@@@@@@@@@@@@"	
					echo					
					fi
					echo "@@@@@@@@@@@@@@@@@@@@@@@@@@ Stream error of $plugname -h @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@" >> $logFile
					$args --help 2>> $logFile
					echo >> $logFile
					let numberOfErrors+=1
																		
				fi
			fi	
		else
		
				let numberOfBlacklisted+=1							
				if [ $verbose -eq 1 ]; then
				echo "øøøøøøøøøøøøøøøøøøøøøøøøøøøøøøøø $1 is blacklisted øøøøøøøøøøøøøøøøøøøøøøøøøøøøøøøø" 
				echo
				fi
				echo "øøøøøøøøøøøøøøøøøøøøøøøøøøøøøøøøøøøøøø $1 is blacklisted ###øøøøøøøøø###############" >> $logFile
			fi


	done	 	

	echo "====== Documented plugin ======" > ${wikiDir}/start.txt 
	for page in $pages
	do
	  echo '  * [[plugins:'$page']]' >> ${wikiDir}/start.txt 
	done


	chown -R apache:apache $wikiDir
	
	echo
	echo "#####################  FAN Dokuwiki updated  ######################"
	echo "$numberOfPages pages created, $numberOfErrors errors occur"
	echo "$numberOfBlacklisted blacklisted plugin(s) not documented"
	echo "Check $logFile for more information"
	echo "###################################################################"
	echo

}



###########################################  The "Main" ##########################################

#rm -Rf $wikiDir


#save the previous logfile
if test -e $logFile; then
	mv $logFile $logFile.old
fi

#variable init
numberOfPages=0


case "$1" in
  -a)
  	rm -Rf $wikiDir
	createWikiPages
	;;

  -d)		 
	rm -Rf $wikiDir$2.txt
	;;

  -h)
  	printHelp
  	;;
  	
  -r)	
	rm -Rf $wikiDir$2.txt
	createWikiPages	
	;;
  -u)	
	 
	createWikiPages	
	;;

   *)
	printHelp
	;;
esac

